@@ -2,13 +2,15 @@ __package__ = "pynibs"
 
 import os
 import copy
+import math
+import h5py
+import shutil
 import random
 import itertools
 import pandas as pd
-from collections import OrderedDict
-
-import h5py
 import numpy as np
+import matplotlib.pyplot as plt
+
 from .exp.exp import read_csv
 from .exp.exp import sort_by_condition
 from .exp.exp import read_exp_stimulations
@@ -22,6 +24,11 @@ from .util import rot_to_quat
 from .util import rotation_matrix_to_euler_angles
 from .util import unique_rows
 
+from collections import OrderedDict
+
+from ortools.constraint_solver import routing_enums_pb2
+from ortools.constraint_solver import pywrapcp
+
 
 def get_coil_dipole_pos(coil_fn, matsimnibs):
     """
@@ -1090,6 +1097,191 @@ def write_coil_pos_hdf5(fn_hdf, centers, m0, m1, m2, datanames=None, data=None,
     f.write('</Xdmf>\n')
     f.close()
 
+
+def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root_path="/0/0/", verbose=False, print_output=False):
+    """
+    Sorts coil positions according to Traveling Salesman problem
+
+    Parameters
+    ----------
+    fn_coil_pos_opt : str
+        Name of .hdf5 file containing the optimal coil position indices
+    fn_coil_pos : str
+        Name of .hdf5 file containing the matsimnibs matrices of all coil positions
+    fn_out_hdf5 : str
+        Name of output .hdf5 file (will be saved in the same format as fn_coil_pos_opt)
+    verbose : bool, optional, default: False
+        Print output messages
+    print_output : bool or str, optional, default: False
+        Print output image as .png file showing optimal path
+
+    Returns
+    -------
+    <file> .hdf5 file containing the sorted optimal coil position indices
+    """
+
+    if verbose:
+        print(f"Loading optimal coil indices from file: {fn_coil_pos_opt}")
+
+    with h5py.File(fn_coil_pos_opt, "r") as f:
+        coil_pos_idx = f[root_path + "coil_seq"][:, 0].astype(int)
+        goal_fun_val = f[root_path + "coil_seq"][:, 1]
+        intensities_opt = f[root_path + "coil_seq"][:, 2]
+
+    if verbose:
+        print(f"Loading all coil positions from file: {fn_coil_pos}")
+
+    with h5py.File(fn_coil_pos, "r") as f:
+        matsimnibs = f["matsimnibs"][:]
+
+    matsimnibs_opt = np.zeros((4, 4, len(coil_pos_idx)))
+
+    for i, idx in enumerate(coil_pos_idx):
+        matsimnibs_opt[:, :, i] = matsimnibs[:, :, idx]
+
+    opt_idx_sort = np.argsort(matsimnibs_opt[0, 3, :])
+    matsimnibs_opt = matsimnibs_opt[:, :, opt_idx_sort]
+
+    unique_idx = np.zeros(len(opt_idx_sort))
+
+    pos_unique = np.unique(matsimnibs_opt[0, 3, :])
+
+    for i, pos in enumerate(pos_unique):
+        unique_idx[matsimnibs_opt[0, 3, :]==pos] = i
+
+    if verbose:
+        print(f"Sorting same positions according to angles ...")
+
+    matsimnibs_opt_sorted_pert = np.zeros(matsimnibs_opt.shape)
+    matsimnibs_opt_sorted = np.zeros(matsimnibs_opt.shape)
+
+    for i in unique_idx:
+        pos = matsimnibs_opt[:, :, unique_idx == i]
+        angles = np.zeros(pos.shape[2])
+
+        if pos.shape[2] > 1:
+            for i_p in range(pos.shape[2]):
+                    dot_prod = np.dot(pos[:3, 0, 0], pos[:3, 0, i_p])
+                    if dot_prod > 1:
+                        dot_prod = 1
+                    elif dot_prod < -1:
+                        dot_prod = -1
+                    angles[i_p] = np.arccos(dot_prod)/np.pi * 180
+
+            angles_sort_idx = np.argsort(angles)
+            pos_sorted = pos[:, :, angles_sort_idx]
+        else:
+            pos_sorted = pos
+
+        matsimnibs_opt_sorted[:, :, unique_idx == i] = pos_sorted
+        matsimnibs_opt_sorted_pert[:, :, unique_idx == i] = pos_sorted
+        matsimnibs_opt_sorted_pert[0, 3, unique_idx == i] += np.arange(len(angles))*1e-2
+
+    coords = matsimnibs_opt_sorted_pert[:3, 3, :].transpose()
+    coords_list = [tuple(c) for c in coords]
+
+    def compute_euclidean_distance_matrix(locations):
+        """Creates callback to return distance between points."""
+        distances = {}
+        for from_counter, from_node in enumerate(locations):
+            distances[from_counter] = {}
+            for to_counter, to_node in enumerate(locations):
+                if from_counter == to_counter:
+                    distances[from_counter][to_counter] = 0
+                else:
+                    # Euclidean distance
+                    distances[from_counter][to_counter] = (int(
+                        math.hypot((from_node[0] - to_node[0]),
+                                   (from_node[1] - to_node[1]))))
+        return distances
+
+    def get_routes(solution, routing, manager):
+        """Get vehicle routes from a solution and store them in an array."""
+        # Get vehicle routes and store them in a two dimensional array whose
+        # i,j entry is the jth location visited by vehicle i along its route.
+        routes = []
+        for route_nbr in range(routing.vehicles()):
+            index = routing.Start(route_nbr)
+            route = [manager.IndexToNode(index)]
+            while not routing.IsEnd(index):
+                index = solution.Value(routing.NextVar(index))
+                route.append(manager.IndexToNode(index))
+            routes.append(route)
+
+        return routes
+
+    def distance_callback(from_index, to_index):
+        """Returns the distance between the two nodes."""
+        # Convert from routing variable Index to distance matrix NodeIndex.
+        from_node = manager.IndexToNode(from_index)
+        to_node = manager.IndexToNode(to_index)
+        return distance_matrix[from_node][to_node]
+
+    # Instantiate the data problem.
+    data = {}
+    # Locations in block units
+    data['locations'] = coords_list
+    data['num_vehicles'] = 1
+    data['depot'] = 0
+
+    # Create the routing index manager.
+    manager = pywrapcp.RoutingIndexManager(len(data['locations']), data['num_vehicles'], data['depot'])
+
+    # Create Routing Model.
+    routing = pywrapcp.RoutingModel(manager)
+    distance_matrix = compute_euclidean_distance_matrix(data['locations'])
+    transit_callback_index = routing.RegisterTransitCallback(distance_callback)
+
+    # Define cost of each arc.
+    routing.SetArcCostEvaluatorOfAllVehicles(transit_callback_index)
+
+    # Setting first solution heuristic.
+    search_parameters = pywrapcp.DefaultRoutingSearchParameters()
+    search_parameters.first_solution_strategy = (routing_enums_pb2.FirstSolutionStrategy.PATH_CHEAPEST_ARC)
+
+    # Solve the problem.
+    if verbose:
+        print(f"Determine optimal order of sequence ...")
+    solution = routing.SolveWithParameters(search_parameters)
+
+    routes = np.array(get_routes(solution, routing, manager)[0])[:-1]
+
+    matsimnibs_opt_sorted = matsimnibs_opt_sorted[:, :, routes]
+
+    idx_global_sorted = np.zeros(len(goal_fun_val))
+    goal_fun_val_sorted = np.zeros(len(goal_fun_val))
+    intensities_opt_sorted = np.zeros(len(intensities_opt))
+
+    for i in range(len(idx_global_sorted)):
+        for j in range(matsimnibs.shape[2]):
+            if (matsimnibs[:, :, j] == matsimnibs_opt_sorted[:, :, i]).all():
+                idx_global_sorted[i] = int(j)
+        goal_fun_val_sorted[i] = goal_fun_val[np.where(int(idx_global_sorted[i]) == coil_pos_idx)[0][0]]
+        intensities_opt_sorted[i] = intensities_opt[np.where(int(idx_global_sorted[i]) == coil_pos_idx)[0][0]]
+
+    # overwrite input file if no output file given
+    outpath = fn_coil_pos_opt
+    if fn_out_hdf5 is not None:
+        outpath = fn_out_hdf5
+        shutil.copy(fn_coil_pos_opt, fn_out_hdf5)
+    
+    if verbose:
+        print(f"Saving output to: {outpath}")
+
+    with h5py.File(outpath, "a") as f:
+        f[root_path+"coil_seq"][:] = np.vstack((idx_global_sorted, goal_fun_val_sorted, intensities_opt_sorted)).transpose() 
+    
+    if print_output:
+        fig = plt.figure()
+        ax = fig.add_subplot(projection='3d')
+        ax.scatter(coords[routes, 0], coords[routes, 1], coords[routes, 2], s=intensities_opt_sorted*11, c="k")
+        ax.plot3D(coords[routes, 0], coords[routes, 1], coords[routes, 2])
+        ax.set_xlabel("x in mm")
+        ax.set_ylabel("y in mm")
+        ax.set_zlabel("z in mm")
+        ax.view_init(elev=40., azim=230)
+        plt.savefig(print_output, dpi=600, transparent=True)
+
 # def check_valid_coil_position_uncertainty(self, problem, coil_position_mean, svd_v):
 #     """ Tests extreme cases of possible coil positions if dipoles are located inside the head.
 #     If an invalid position is found, the parameter space is reduced by shrinking the limits.
