@@ -1139,15 +1139,15 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
 
     Returns
     -------
-    p2p_array : np.array of float [Nx1]
+    p2p_array : np.array of float [N_stim]
         Peak to peak values of N sweeps
     time_mep_lst : list of datetime.timedelta
         Mep-timestamps
-    mep_raw_data : np.array of float [N x N_samples]
+    mep_raw_data : np.array of float [N_channel x N_stim x N_samples]
         Raw (unfiltered) MEP data
-    mep_filt_data : np.array of float [N x N_samples]
+    mep_filt_data : np.array of float [N_channel x N_stim x N_samples]
         Filtered MEP data (Butterworth lowpass filter)
-    time : np.array of float [N]
+    time : np.array of float [N_samples]
         Time axis corresponding to MEP data
     """
     # convert pulse time to datetime object in case of "delta"
