@@ -5,9 +5,10 @@ import scipy
 import os
 import time
 import h5py
-import nibabel
+import nibabel as nib
 import trimesh
 from pynibs.util import sample_sphere
+from simnibs import SIMNIBSDIR, mesh_io, transformations
 
 
 def get_mask(areas, fn_annot, fn_inflated_fs, fn_out):
@@ -31,7 +32,7 @@ def get_mask(areas, fn_annot, fn_inflated_fs, fn_out):
         fn_out.overlay file of freesurfer mask
     """
     # read annotation file
-    vertices, colortable, label = nibabel.freesurfer.io.read_annot(fn_annot)
+    vertices, colortable, label = nib.freesurfer.io.read_annot(fn_annot)
 
     # convert label from numpy bytes str to str
     label = [str(l.astype(str)) for l in label]
@@ -114,12 +115,12 @@ def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_t
 
         stop = time.time()
         print('Determining element index of point: {:s}/{:d} ({:d}/{:d}) \t [{:1.2f} sec] \t {:1.2f}%' \
-            .format(str(i).zfill(int(np.floor(np.log10(N_points_total)) + 1)),
-                    N_points_total,
-                    i_local,
-                    len(chunk),
-                    stop - start,
-                    float(i_local) / (N_points_total / N_CPU) * 100.0))
+              .format(str(i).zfill(int(np.floor(np.log10(N_points_total)) + 1)),
+                      N_points_total,
+                      i_local,
+                      len(chunk),
+                      stop - start,
+                      float(i_local) / (N_points_total / N_CPU) * 100.0))
     return tet_idx_local
 
 
@@ -159,24 +160,28 @@ def load_roi_surface_obj_from_hdf5(fname):
             # read data from .hdf5 file and pass it to object
             for j in range(len(data_label)):
                 if type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == np.ndarray and \
-                    "S" in str(f[f'roi_surface/{roi_idx}/{data_label[j]}'].dtype) and \
+                        "S" in str(f[f'roi_surface/{roi_idx}/{data_label[j]}'].dtype) and \
                         len(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) > 1:
-                    expr = "roi[roi_idx]." + data_label[j] + "= list(f[f'roi_surface/{roi_idx}/{data_label[j]}'][:].astype(str))"
+                    expr = "roi[roi_idx]." + data_label[
+                        j] + "= list(f[f'roi_surface/{roi_idx}/{data_label[j]}'][:].astype(str))"
                 elif type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == np.bytes_ and \
                         "S" in str(f[f'roi_surface/{roi_idx}/{data_label[j]}'].dtype):
-                    expr = "roi[roi_idx]." + data_label[j] + "= str(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()].astype(str))"
+                    expr = "roi[roi_idx]." + data_label[
+                        j] + "= str(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()].astype(str))"
 
                 # bytes string
                 elif type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == bytes:
-                    expr = "roi[roi_idx]." + data_label[j] + "= f[f'roi_surface/{roi_idx}/{data_label[j]}'][()].decode('utf-8')"
+                    expr = "roi[roi_idx]." + data_label[
+                        j] + "= f[f'roi_surface/{roi_idx}/{data_label[j]}'][()].decode('utf-8')"
 
                 # list of bytes string
                 elif type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == list and \
-                            len(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) > 1:
-                    expr = "roi[roi_idx]." + data_label[j] + "= list(f[f'roi_surface/{roi_idx}/{data_label[j]}'][:].astype(str))"
+                        len(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) > 1:
+                    expr = "roi[roi_idx]." + data_label[
+                        j] + "= list(f[f'roi_surface/{roi_idx}/{data_label[j]}'][:].astype(str))"
 
                 elif type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == np.float64 or \
-                    type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == np.int64:
+                        type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == np.int64:
                     expr = "roi[roi_idx]." + data_label[j] + "= f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]"
                 else:
                     expr = "roi[roi_idx]." + data_label[j] + "= np.array(f[f'roi_surface/{roi_idx}/{data_label[j]}'])"
@@ -396,17 +401,18 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
 
     for i in range(len(gm_surf_fname)):
         if gm_surf_fname[i] is not None:
-            points_gm[i], con_gm[i] = nibabel.freesurfer.read_geometry(os.path.join(mesh_folder, gm_surf_fname[i]))
+            points_gm[i], con_gm[i] = nib.freesurfer.read_geometry(os.path.join(mesh_folder, gm_surf_fname[i]))
             con_gm[i] = con_gm[i] + max_idx_gm
             max_idx_gm = max_idx_gm + points_gm[i].shape[0]  # np.max(con_gm[i]) + 2
 
         if wm_surf_fname[i] is not None:
-            points_wm[i], con_wm[i] = nibabel.freesurfer.read_geometry(os.path.join(mesh_folder, wm_surf_fname[i]))
+            points_wm[i], con_wm[i] = nib.freesurfer.read_geometry(os.path.join(mesh_folder, wm_surf_fname[i]))
             con_wm[i] = con_wm[i] + max_idx_wm
             max_idx_wm = max_idx_wm + points_wm[i].shape[0]  # np.max(con_wm[i]) + 2
 
         if midlayer_surf_fname[i] is not None:
-            points_mid[i], con_mid[i] = nibabel.freesurfer.read_geometry(os.path.join(mesh_folder, midlayer_surf_fname[i]))
+            points_mid[i], con_mid[i] = nib.freesurfer.read_geometry(
+                os.path.join(mesh_folder, midlayer_surf_fname[i]))
             con_mid[i] = con_mid[i] + max_idx_mid
             max_idx_mid = max_idx_mid + points_mid[i].shape[0]  # np.max(con_wm[i]) + 2
 
@@ -476,15 +482,15 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
 
     else:
         # read mask from freesurfer mask file
-        mask = nibabel.freesurfer.mghformat.MGHImage.from_filename(os.path.join(mesh_folder, fn_mask)).dataobj[:]
+        mask = nib.freesurfer.mghformat.MGHImage.from_filename(os.path.join(mesh_folder, fn_mask)).dataobj[:]
         roi_mask_idx = np.where(mask > 0.5)
 
     # redefine connectivity matrix for cropped points (reindexing)
     # get row index where all points are lying inside ROI
     if not (x_roi_wb and y_roi_wb and z_roi_wb):
-        con_row_idx = [i for i in range(con.shape[0]) if len(np.intersect1d(con[i, ], roi_mask_idx)) == 3]
+        con_row_idx = [i for i in range(con.shape[0]) if len(np.intersect1d(con[i,], roi_mask_idx)) == 3]
         # crop connectivity matrix to ROI
-        con_cropped = con[con_row_idx, ]
+        con_cropped = con[con_row_idx,]
     else:
         con_cropped = con
 
@@ -493,10 +499,10 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
     con_cropped_reform = np.reshape(point_idx_after, (con_cropped.shape[0], con_cropped.shape[1]))
 
     # crop points to ROI
-    surface_points_middle = surface_points_middle[point_idx_before, ]
+    surface_points_middle = surface_points_middle[point_idx_before,]
     if layer == 3:
-        surface_points_upper = surface_points_upper[point_idx_before, ]
-        surface_points_lower = surface_points_lower[point_idx_before, ]
+        surface_points_upper = surface_points_upper[point_idx_before,]
+        surface_points_lower = surface_points_lower[point_idx_before,]
 
     # refine
     if refine:
@@ -579,7 +585,11 @@ class RegionOfInterestSurface:
     tri_center_coord_low : nparray [N_tri x 3]
         Coordinates of roi triangle center of lower surface
     fn_mask : string
-        Filename for freesurfer mask. If given, this is used instead of *_ROIs
+        Filename for .mgh mask in subject space.
+    fn_mask_avg : string
+        Filename for .mgh mask in fsaverage space.
+    fn_mask : string
+        Filename for .nii or .nii.gz mask.
     X_ROI : list of float
         Region of interest [Xmin, Xmax], whole X range if empty [0,0] or None
         (left - right)
@@ -589,6 +599,12 @@ class RegionOfInterestSurface:
     Z_ROI : list of float
         Region of interest [Zmin, Zmax], whole Z range if empty [0,0] or None
         (inferior - superior)
+    template : str
+        'MNI', 'fsaverage', 'subject
+    center : list of float
+        Center coordinates for spherical ROI in self.template space
+    radius : float
+        Radius in [mm] for spherical ROI
     gm_surf_fname : str or list of str
         Filename(s) of GM surface generated by freesurfer (lh and/or rh)
         (e.g. in mri2msh: .../fs_ID/surf/lh.pial)
@@ -629,11 +645,17 @@ class RegionOfInterestSurface:
         self.tri_center_coord_low = []
 
         self.fn_mask = []
+        self.fn_mask_avg = None
+        self.fn_mask_mni = None
 
         self.X_ROI = []
         self.Y_ROI = []
         self.Z_ROI = []
 
+        self.center_mni = None
+        self.center_sub = None
+        self.radius = None
+
         self.gm_surf_fname = []
         self.wm_surf_fname = []
         self.midlayer_surf_fname = []
@@ -669,11 +691,12 @@ class RegionOfInterestSurface:
         target_proj = self.node_coord_mid[target_node_coord_mid]
 
         if verbose:
-            print(f"Projected {target} to {target_proj} (Dist: {np.linalg.norm(target-target_proj):2.2f}mm)")
+            print(f"Projected {target} to {target_proj} (Dist: {np.linalg.norm(target - target_proj):2.2f}mm)")
 
         return target_proj
 
-    def make_GM_WM_surface(self, gm_surf_fname, wm_surf_fname, midlayer_surf_fname, mesh_folder, delta=0.5,
+    def make_GM_WM_surface(self, gm_surf_fname=None, wm_surf_fname=None, midlayer_surf_fname=None, mesh_folder=None,
+                           delta=0.5,
                            X_ROI=None, Y_ROI=None, Z_ROI=None,
                            layer=1,
                            fn_mask=None, refine=False):
@@ -684,16 +707,16 @@ class RegionOfInterestSurface:
         Parameters
         ----------
         gm_surf_fname : str or list of str
-            filename(s) of GM surface generated by freesurfer (lh and/or rh)
-            (e.g. in mri2msh: .../fs_ID/surf/lh.pial)
+            Filename(s) of GM FreeSurfer surface(s)  (lh and/or rh).
+            Either relative to mesh_folder (fs_ID/surf/lh.pial) or absolute (/full/path/to/lh.pial)
         wm_surf_fname : str or list of str
-            filename(s) of WM surface generated by freesurfer (lh and/or rh)
-            (e.g. in mri2msh: .../fs_ID/surf/lh.white)
+            Filename(s) of WM FreeSurfer surface(s) (lh and/or rh)
+            Either relative to mesh_folder (fs_ID/surf/lh.white) or absolute (/full/path/to/lh.white)
         midlayer_surf_fname : str or list of str
-            filename(s) of midlayer surface generated by headreco (lh and/or rh)
-            (e.g. in headreco: .../fs_ID/surf/lh.central) (after conversion)
+            Filename(s) of midlayer surface (lh and/or rh)
+            Either relative to mesh_folder (fs_ID/surf/lh.central) or absolute (/full/path/to/lh.central)
         mesh_folder : str
-            Path of mesh (parent directory)
+            Root folder of mesh, Needed if paths above are given relative, or refine=True
         [defunct] m2m_mat_fname : str
             Filename of mri2msh transformation matrix
             (e.g. in mri2msh: .../m2m_ProbandID/MNI2conform_6DOF.mat)
@@ -714,8 +737,8 @@ class RegionOfInterestSurface:
             Define the number of layers:
             - 1: one layer
             - 3: additionally upper and lower layers are generated around the central midlayer
-        fn_mask : string
-            Filename for Freesurfer mask. If given, this is used instead of *_ROIs
+        fn_mask : str
+            Filename for FreeSurfer .mgh mask.
         refine : bool, optional, default: False
             Refine ROI by splitting elements
 
@@ -795,17 +818,23 @@ class RegionOfInterestSurface:
 
         for i in range(len(gm_surf_fname)):
             if gm_surf_fname[i] is not None:
-                points_gm[i], con_gm[i] = nibabel.freesurfer.read_geometry(os.path.join(mesh_folder, gm_surf_fname[i]))
+                if not gm_surf_fname[i].startswith(os.sep):
+                    gm_surf_fname[i] = os.path.join(mesh_folder, gm_surf_fname[i])
+                points_gm[i], con_gm[i] = nib.freesurfer.read_geometry(gm_surf_fname[i])
                 con_gm[i] = con_gm[i] + max_idx_gm
                 max_idx_gm = max_idx_gm + points_gm[i].shape[0]  # np.max(con_gm[i]) + 2
 
             if wm_surf_fname[i] is not None:
-                points_wm[i], con_wm[i] = nibabel.freesurfer.read_geometry(os.path.join(mesh_folder, wm_surf_fname[i]))
+                if not wm_surf_fname[i].startswith(os.sep):
+                    wm_surf_fname[i] = os.path.join(mesh_folder, wm_surf_fname[i])
+                points_wm[i], con_wm[i] = nib.freesurfer.read_geometry(wm_surf_fname[i])
                 con_wm[i] = con_wm[i] + max_idx_wm
                 max_idx_wm = max_idx_wm + points_wm[i].shape[0]  # np.max(con_wm[i]) + 2
 
             if midlayer_surf_fname[i] is not None:
-                points_mid[i], con_mid[i] = nibabel.freesurfer.read_geometry(os.path.join(mesh_folder, midlayer_surf_fname[i]))
+                if not midlayer_surf_fname[i].startswith(os.sep):
+                    midlayer_surf_fname[i] = os.path.join(mesh_folder, midlayer_surf_fname[i])
+                points_mid[i], con_mid[i] = nib.freesurfer.read_geometry(midlayer_surf_fname[i])
                 con_mid[i] = con_mid[i] + max_idx_mid
                 max_idx_mid = max_idx_mid + points_mid[i].shape[0]  # np.max(con_wm[i]) + 2
 
@@ -874,16 +903,24 @@ class RegionOfInterestSurface:
             roi_mask_idx = np.where(roi_mask_bool)
 
         else:
+            if X_ROI is not None or Y_ROI is not None or Z_ROI is not None:
+                raise ValueError(f"Either provide X_ROI, Y_ROI, Z_ROI or fn_mask, not both.")
+
             # read mask from freesurfer mask file
-            mask = nibabel.freesurfer.mghformat.MGHImage.from_filename(os.path.join(mesh_folder, fn_mask)).dataobj[:]
+            if not fn_mask.startswith(os.sep):
+                fn_mask = os.path.join(mesh_folder, fn_mask)
+            if fn_mask.endswith('.mgh') or fn_mask.endswith('.mgz'):
+                mask = nib.freesurfer.mghformat.MGHImage.from_filename(fn_mask).dataobj[:]
+            else:
+                mask = nib.freesurfer.read_morph_data(fn_mask)[:, np.newaxis, np.newaxis]
             roi_mask_idx = np.where(mask > 0.5)
 
         # redefine connectivity matrix for cropped points (reindexing)
         # get row index where all points are lying inside ROI
         if not (x_roi_wb and y_roi_wb and z_roi_wb):
-            con_row_idx = [i for i in range(con.shape[0]) if len(np.intersect1d(con[i, ], roi_mask_idx)) == 3]
+            con_row_idx = [i for i in range(con.shape[0]) if len(np.intersect1d(con[i,], roi_mask_idx)) == 3]
             # crop connectivity matrix to ROI
-            con_cropped = con[con_row_idx, ]
+            con_cropped = con[con_row_idx,]
         else:
             con_cropped = con
 
@@ -892,10 +929,10 @@ class RegionOfInterestSurface:
         con_cropped_reform = np.reshape(point_idx_after, (con_cropped.shape[0], con_cropped.shape[1]))
 
         # crop points to ROI
-        surface_points_middle = surface_points_middle[point_idx_before, ]
+        surface_points_middle = surface_points_middle[point_idx_before,]
         if self.layer == 3:
-            surface_points_upper = surface_points_upper[point_idx_before, ]
-            surface_points_lower = surface_points_lower[point_idx_before, ]
+            surface_points_upper = surface_points_upper[point_idx_before,]
+            surface_points_lower = surface_points_lower[point_idx_before,]
 
         # refine
         if refine:
@@ -1023,31 +1060,33 @@ class RegionOfInterestSurface:
         """
 
         # load sphere surface
-        sphere_coords, sphere_faces = nibabel.freesurfer.io.read_geometry(fn_sphere)
+        sphere_coords, sphere_faces = nib.freesurfer.io.read_geometry(fn_sphere)
 
         # sample sphere equally
         r = np.linalg.norm(sphere_coords[0, :])
-        A0 = 4*np.pi*r**2
+        A0 = 4 * np.pi * r ** 2
 
-        sphere_points_sampled = sample_sphere(n_points=int(np.ceil(A0/dist**2) // 2 * 2 + 1), r=np.linalg.norm(sphere_coords[0, :]))
+        sphere_points_sampled = sample_sphere(n_points=int(np.ceil(A0 / dist ** 2) // 2 * 2 + 1),
+                                              r=np.linalg.norm(sphere_coords[0, :]))
 
-        #from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
-        #import matplotlib.pyplot as plt
+        # from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
+        # import matplotlib.pyplot as plt
 
-
-        #fig = plt.figure()
-        #ax = fig.add_subplot(111, projection='3d')
-        #ax.scatter(sphere_points_sampled[:, 0], sphere_points_sampled[:, 1], sphere_points_sampled[:, 2])
-        #triangular_mesh(sphere_coords[:, 0], sphere_coords[:, 1], sphere_coords[:, 2], sphere_faces, tube_radius=None, line_width=0.01, scale_factor=0.0001)
+        # fig = plt.figure()
+        # ax = fig.add_subplot(111, projection='3d')
+        # ax.scatter(sphere_points_sampled[:, 0], sphere_points_sampled[:, 1], sphere_points_sampled[:, 2])
+        # triangular_mesh(sphere_coords[:, 0], sphere_coords[:, 1], sphere_coords[:, 2], sphere_faces, tube_radius=None, line_width=0.01, scale_factor=0.0001)
 
         # read mask from freesurfer mask file
-        mask = nibabel.freesurfer.mghformat.MGHImage.from_filename(os.path.join(self.mesh_folder, self.fn_mask)).dataobj[:].flatten()
+        mask = nib.freesurfer.mghformat.MGHImage.from_filename(
+            os.path.join(self.mesh_folder, self.fn_mask)).dataobj[:].flatten()
         roi_mask_idx = np.where(mask > 0.5)
 
-        con_row_idx = [i for i in range(sphere_faces.shape[0]) if len(np.intersect1d(sphere_faces[i, ], roi_mask_idx)) == 3]
+        con_row_idx = [i for i in range(sphere_faces.shape[0]) if
+                       len(np.intersect1d(sphere_faces[i,], roi_mask_idx)) == 3]
 
         # crop connectivity matrix to ROI
-        con_cropped = sphere_faces[con_row_idx, ]
+        con_cropped = sphere_faces[con_row_idx,]
 
         p1_tri = sphere_coords[con_cropped[:, 0], :]
         p2_tri = sphere_coords[con_cropped[:, 1], :]
@@ -1147,7 +1186,7 @@ class RegionOfInterestVolume:
             roi_mask_idx = np.where(roi_mask_bool)[0]
 
         elif volume_type == 'sphere':
-            roi_mask_bool = np.linalg.norm(msh.points-np.array(X_ROI), axis=1) <= Y_ROI
+            roi_mask_bool = np.linalg.norm(msh.points - np.array(X_ROI), axis=1) <= Y_ROI
             roi_mask_idx = np.where(roi_mask_bool)[0]
 
         else:
@@ -1157,19 +1196,21 @@ class RegionOfInterestVolume:
 
         # crop connectivity matrices of tetrahedra and triangles to ROI
         tet_con_row_idx = [i for i in range(msh.tetrahedra.shape[0]) if
-                           len(np.intersect1d(msh.tetrahedra[i, ], roi_mask_idx)) == 3]
-        tet_con_cropped = msh.tetrahedra[tet_con_row_idx, ]
+                           len(np.intersect1d(msh.tetrahedra[i,], roi_mask_idx)) == 3]
+        tet_con_cropped = msh.tetrahedra[tet_con_row_idx,]
 
         tri_con_row_idx = [i for i in range(msh.triangles.shape[0]) if
-                           len(np.intersect1d(msh.triangles[i, ], roi_mask_idx)) == 3]
-        tri_con_cropped = msh.triangles[tri_con_row_idx, ]
+                           len(np.intersect1d(msh.triangles[i,], roi_mask_idx)) == 3]
+        tri_con_cropped = msh.triangles[tri_con_row_idx,]
 
         # evaluate new indices of cropped connectivity matrices of tetrahedra and triangles (starts from 0)
         tet_point_idx_before, tet_point_idx_after = np.unique(tet_con_cropped, return_inverse=True)
-        self.tet_node_number_list = np.reshape(tet_point_idx_after, (tet_con_cropped.shape[0], tet_con_cropped.shape[1]))
+        self.tet_node_number_list = np.reshape(tet_point_idx_after,
+                                               (tet_con_cropped.shape[0], tet_con_cropped.shape[1]))
 
         tri_point_idx_before, tri_point_idx_after = np.unique(tri_con_cropped, return_inverse=True)
-        self.tri_node_number_list = np.reshape(tri_point_idx_after, (tri_con_cropped.shape[0], tri_con_cropped.shape[1]))
+        self.tri_node_number_list = np.reshape(tri_point_idx_after,
+                                               (tri_con_cropped.shape[0], tri_con_cropped.shape[1]))
 
         self.tet_idx_node_coord = None
         self.tet_idx_tetrahedra_center = np.array(tet_con_row_idx)
@@ -1193,8 +1234,6 @@ class RegionOfInterestVolume:
     #     a=1
 
 
-
-
 # def determine_element_idx_in_mesh_old(self, fname, points, N_MAX_CPU=20):
 #     """
 #     Determine the element indices where the points_out are lying and safe the results to a .txt file.
@@ -1297,4 +1336,167 @@ class RegionOfInterestVolume:
 #     np.savetxt(fname, tet_idx, '%d')
 #     print 'Saved element indices of points_out in {}'.format(fname)
 #
-#     return tet_idx
\ No newline at end of file
+#     return tet_idx
+
+
+def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
+                      thresh_by_nii=True, val_in=1, val_out=0):
+    """
+    Computes a spherical ROI for a given Nifti image (defaults to simnibs MNI T1 tissue). The ROI area is defined
+    in nifti coordinates.
+    By default, everything inside the ROI is set to 1, areas outside = 0.
+    The ROI is further thresholded by the nifti.
+    A nib.Nifti image is returned and optionally saved.
+
+    Parameters
+    ----------
+    center : array-like
+        X, Y, Z coordinates in nifti space
+    radius : float
+        radius of sphere
+    nii : string or nib.nifti1.Nifti1Image, optional
+        The nifti image to work with.
+    out_fn : string, optional
+        If provided, sphere ROI image is saved here
+
+    Returns
+    -------
+    sphere_img :  nib.nifti1.Nifti1Image
+    sphere_img : <file>, optional
+
+    Other Parameters
+    ----------------
+    thresh_by_nii : bool, optional
+        Mask sphere by nii != 0
+    val_in : float, optional
+        Value within ROI
+    val_out : float, optional
+        Value outside ROI
+
+    Raises
+    ------
+    ValueError
+        If the final ROI is empty.
+    """
+
+    # load image for affine and tissue borders
+    if nii is None:
+        mni_fn = f"{SIMNIBSDIR}/resources/templates/spmprior_tissue.nii"
+        nii = nib.load(mni_fn)
+    elif isinstance(nii, str):
+        nii = nib.load(nii)
+
+    center = np.atleast_1d(center)
+    assert center.shape == (3,)
+
+    roi_mni_data = np.zeros_like(nii.get_fdata())
+    roi_mni_data[:] = val_out
+
+    center = np.atleast_1d(center)
+
+    # compute spherical roi around nifti coordinates
+    # do this in original coordinates to have correct mm radius
+    for x in range(center[0] - radius, center[0] + radius):
+        for y in range(center[1] - radius, center[1] + radius):
+            for z in range(center[2] - radius, center[2] + radius):
+                # set to val_in if within radius
+                if np.linalg.norm(center - np.array([x, y, z])) < radius:
+                    vox = nib.affines.apply_affine(np.linalg.inv(nii.affine), [x, y, z]).astype(int)
+                    # print(f"{x}, {y}, {z} -- {vox}: {np.linalg.norm(center - np.array([x,y,z]))}")
+                    roi_mni_data[vox[0], vox[1], vox[2]] = val_in
+
+    # check if there's something left in the ROI
+    if np.sum(roi_mni_data) == 0:
+        raise ValueError(f"No tissue found in {radius} mm sphere around {center}.")
+
+    # threshold roi by tissue
+    if thresh_by_nii:
+        roi_mni_data[nii.get_fdata() == 0] = 0
+
+    if out_fn is not None:
+        # make new roi image and save file
+        roi_mni_img = nib.Nifti1Image(roi_mni_data, nii.affine)
+        nib.save(roi_mni_img, out_fn)
+
+
+def clean_roi(img, vox_thres=.5, fn_out=None):
+    """
+    Remove values < vox thres from image.
+
+    Parameters
+    ----------
+    img : str or nibabel.nifti1.Nifti1Image
+    vox_thres : float, optional
+    fn_out : str
+
+    Returns
+    -------
+    img_thres : nibabel.nifti1.Nifti1Image
+    img_thres : <file>
+        If fn_out is specified, thresholded image is saved here
+    """
+    # threshold subject space image to remove speckles
+    if isinstance(img, str):
+        nii = nib.load(img)
+    else:
+        nii = img
+    data = nii.get_fdata()
+    data[(data < vox_thres)] = 0
+    nii = nib.Nifti1Image(data, nii.affine)
+    if fn_out is not None:
+        nib.save(nii, fn_out)
+    return nii
+
+
+def nii2msh(mesh, m2m_dir, nii, out_folder, hem, out_fsaverage=False, roi_name='ROI'):
+    """
+    Transform a nifti ROI image to subject space .mgh file.
+
+    Parameters
+    ----------
+    mesh : simnibs.Mesh or str
+    m2m_dir : str
+    nii : nibabel.nifti1.Nifti1Image or str
+    out_folder : str
+    hem : str
+        'lh' or 'rh'
+    out_fsaverage : bool
+
+    Returns
+    -------
+    roi : file
+        f"{out_folder}/{hem}.mesh.central.{roi_name}"
+
+    Other Parameters
+    ----------------
+    roi_name : str
+        How to name the ROI
+    """
+    if isinstance(mesh, str):
+        mesh = mesh_io.read_msh(mesh)
+    if isinstance(nii, str):
+        nii = nib.load(nii)
+    assert hem in ['lh', 'rh'], f"hem argument must be one of ('lh','rh). You specified hem={hem}."
+
+    fn_mesh_out = os.path.join(out_folder, 'mesh.msh')
+    vol = nii.dataobj
+    affine = nii.affine
+
+    # Interpolating data in NifTI file to mesh
+    # nd = mesh_io.NodeData.from_data_grid(mesh, vol, affine, 'from_volume')
+
+    # meshify
+    ed = mesh_io.ElementData.from_data_grid(mesh, vol, affine, roi_name)
+    mesh.nodedata = []
+    mesh.elmdata = [ed]
+    mesh_io.write_msh(mesh, fn_mesh_out)
+
+    # trasform to midlayer
+    if out_fsaverage:
+        out_fsaverage = out_folder
+    else:
+        out_fsaverage = None
+    transformations.middle_gm_interpolation(fn_mesh_out, m2m_folder=m2m_dir,
+                                            out_folder=out_folder, out_fsaverage=out_folder)
+
+    return nib.freesurfer.read_morph_data(os.path.join(out_folder, f"{hem}.mesh.central.{roi_name}"))
\ No newline at end of file
