@@ -299,12 +299,13 @@ class Mesh:
     """"
     Mesh class to initialize default attributes.
     """
+
     def __init__(self, mesh_name, subject_id, subject_folder):
         self.subject_id = subject_id
         self.subject_folder = subject_folder
         self.name = mesh_name
         self.info = None
-        self.approach = None
+        self.approach = None # 'mri2mesh', 'headreco'
         self.mri_idx = None
 
         # default parameters
@@ -315,7 +316,8 @@ class Mesh:
         self.fn_mri_conform = f"{subject_id}_T1fs_conform.nii.gz"
         self.fn_lh_midlayer = f"fs_{subject_id}/surf/lh.central"
         self.fn_rh_midlayer = f"fs_{subject_id}/surf/rh.central"
-        self.vertex_density = 1.0
+        self.vertex_density = 1.0 # headreco
+        self.numvertices = 100000 # mri2mesh
 
         # refinement parameters
         self.center = None
@@ -348,8 +350,8 @@ class Mesh:
             self.fn_mri_conform = f"{self.subject_id}_T1fs_conform.nii.gz"
             self.fn_lh_midlayer = f"fs_{self.subject_id}/surf/lh.central"
             self.fn_rh_midlayer = f"fs_{self.subject_id}/surf/rh.central"
-        else:
-            raise NotImplementedError(f"Approach {approach} not implemented.")
+        # else:
+        #     # raise NotImplementedError(f"Approach {approach} not implemented.")
 
     def write_to_hdf5(self, fn_hdf5, check_file_exist=False, verbose=False):
         """
@@ -374,7 +376,7 @@ class Mesh:
         Print self information.
         """
         n_left, n_right = int(32 - np.floor((len(self.name) + 10) / 2)), int(32 - np.ceil((len(self.name) + 10) / 2))
-        n_left, n_right = np.max(n_left,0), np.max(n_right,0)
+        n_left, n_right = np.max(n_left, 0), np.max(n_right, 0)
         print("    " + "=" * n_left + f"  Mesh {self.name}:  " + "=" * n_right)
         print("\t" + json.dumps(self.__dict__, sort_keys=False, indent="\t", ))
         print("    " + "=" * 64 + "\n")
@@ -382,15 +384,17 @@ class Mesh:
 
 class ROI:
     """"
-    Region of interest class to initialize default attribrutes.
+    Region of interest class to initialize default attributes.
 
     """
+
     def __init__(self, subject_id, roi_name, mesh_name):
         self.subject_id = subject_id
         self.name = roi_name
         self.mesh_name = mesh_name
-        self.type = None
+        self.type = None  # 'surface' or 'volume'
         self.info = None
+        self.template = None  # None, 'MNI', 'fsaverage', 'subject'
         self.gm_surf_fname = None
         self.wm_surf_fname = None
         self.midlayer_surf_fname = None
@@ -399,6 +403,8 @@ class ROI:
         self.X_ROI = None
         self.Y_ROI = None
         self.Z_ROI = None
+        self.center = None
+        self.radius = None
         self.layer = 3
         self.fn_mask = None
         self.fn_mask_avg = None
@@ -428,7 +434,7 @@ class ROI:
         Print self information.
         """
         n_left, n_right = int(32 - np.floor((len(self.name) + 10) / 2)), int(32 - np.ceil((len(self.name) + 10) / 2))
-        n_left, n_right = np.max(n_left,0), np.max(n_right,0)
+        n_left, n_right = np.max(n_left, 0), np.max(n_right, 0)
         print("    " + "=" * n_left + f"  ROI {self.name}:  " + "=" * n_right)
         print("\t" + json.dumps(self.__dict__, sort_keys=False, indent="\t", ))
         print("    " + "=" * 64 + "\n")
